require_relative "../base"
require 'sys/proctable'

module BaselineRedAgent
  module Plugin
    module System
      class Processes < BaselineRedAgent::Plugin::Base
        include Sys

        ALLOWED_PROCESSES_REGEX = /(ruby|puma|passenger|unicorn)/i

        def call
          items = []

          ProcTable.ps.each do |process|
            next unless process.comm =~ ALLOWED_PROCESSES_REGEX

            items << ["system.processes.rss", process.respond_to?(:pctmem) ? process.pctmem : 0,
                      { "name" => "#{BaselineRedAgent.options[:host]} - #{process.comm} (#{process.pid.to_s})" ,
                        "process" => {
                            "name" => process.name,
                            "pid"  => process.pid.to_s,
                            "command_line" => process.comm,
                        }
                      }]

            items << ["system.processes.cpu", process.respond_to?(:pctcpu) ? process.pctcpu : 0,
                      { "name" => "#{BaselineRedAgent.options[:host]} - #{process.comm} (#{process.pid.to_s})",
                        "process" => {
                            "name" => process.name,
                            "pid"  => process.pid.to_s,
                            "command_line" => process.comm,
                        }
                      }]
          end

          items
        end
      end
    end
  end
end

BaselineRedAgent.logger.info "Loading Process monitoring."
